/*
 *  NodeColoring.java
 *
 *  Copyright (C) 1997 Esa Hyyti
 *
 *  This class determines a node coloring of the given graph.
 *  Graph is represented in binary (neighbour) matrix.
 */


class NodeColoring 
{
    static boolean DEBUG = false;
    
    BinaryMatrix c;
    int color[] = null;
    int num;

    public NodeColoring( BinaryMatrix c ) 
    {
        this.c = c;
        color = new int[c.np];
        greedyColor();   // default
    }

    public void print() 
    {
        int i;

        for( i=0; i<c.np; i++ )
            System.out.print( color[i] + " " );
        System.out.println( );
    }
    
    /*
     *  Assigns colors to given graph with Greedy Algorithm
     */
    public int greedyColor( ) 
    {
        int i,j,k;
        int degree[] = new int[c.np];
        int order[]  = new int[c.np];

	if (DEBUG)
            c.print();

        for (i=0; i<c.np; i++ )            /* calculate degree of nodes */
        {
            degree[i] = 0;
            for (j=0; j<c.np; j++ )
                if ( c.m[i][j] )
                    degree[i]++;
            color[i]  = -1;
        }
        
        for (i=0; i<c.np; i++ )            /* create ordering */
        {
            for( j=0; j<i; j++ )
                if ( degree[i] > degree[ order[j] ] )
                    break;
            /* move old ones one up */
            for (k=i; k>j; k-- )
                order[k] = order[k-1];
            order[j] = i;
        }

        /* assign colors in that order */
        num = 1;
        color[order[0]] = 1;
        for (i=1; i<c.np; i++ )
        {
            int n1 = order[i];
            int col;
            
            /* try old ones first, numbering starts from 1! */
            for( col=1; col<=num; col++ )
            {
                for ( j=0; j<c.np; j++ )
                    if ( c.m[n1][j] && color[j] == col )
                        break;
                if ( j >= c.np )
                    break;
            }
            color[n1] = col;
            if ( col > num )
                num = col;
        }

        return num;
    }
}
