/*
 *  Routes.java
 *
 *  Copyright (C) 1997 Esa Hyyti
 *
 *  Defines a set of possible routes between a pair of nodes
 *
 */

import java.util.Vector;

class Routes 
{
    static boolean DEBUG = false;
    
    public int from;
    public int to;
    public Vector routes;   /* entries are Route */
    
    boolean status;
    
    public boolean status() 
    {
        return status;
    }

    public Route getRoute( int index ) 
    {
        Route r = (Route)(routes.elementAt(index));
        return r;
    }
    
    public Routes( BinaryMatrix m, int a, int b ) 
    {
        Route route;
        int distance[];
        int i,j,d;

        routes = new Vector();
        from = a;
        to   = b;

        if ( a<0 || a>=m.np || b<0 || b>=m.np || m.np < 1 )
        {
            status = false;
            return;
        }
        distance = new int[m.np];
        
        for ( i=0; i<m.np; i++ )
            distance[i] = -1;

        d = 0;
        distance[a] = d;

        boolean progress = true;
        
        while ( distance[b] == -1 && progress )
        {
            progress = false;
            for ( i=0; i<m.np; i++ )     /* neighbours */
            {
                
                if ( distance[i] == d )
                {
                    for ( j=0; j<m.np; j++ )
                        if ( m.m[i][j] && distance[j] == -1 )
                        {
                            distance[j] = d + 1;
                            progress = true;
                        }
                    
                }
            }
            d++;
        }

        if ( distance[b] == -1 )    /* no route */
        {
            status = false;
            return;
        }
        
        // /* reject those which were not there yet */
        // for ( i=0; i<np; i++ )
        // {
        //     if ( distance[i] == d && i!=b )
        //         distance[i] = -1;
        // }
        
        /* then go backward and write down the routes */
        d = d + 1;
        route = new Route( d );
        routes.addElement( route );
        i = d-1;
        route.node[i] = b;
        while ( i>0 )
        {
            int r;
            
            i = i - 1;
            r = 0;
            while ( r < routes.size() )
            {
                int pos = 0;
                
                route = (Route)routes.elementAt(r);
                
                /* find next one */
                for( j=0; j<m.np; j++ )
                    if ( distance[j] == i && m.m[route.node[i+1]][j] )
                    {
                        if ( pos == 0 )
                            route.node[i] = j;
                        else
                        {
                            Route newroute = new Route(d);
                            for( int k=d-1; k>i; k-- )
                                newroute.node[k] = route.node[k];
                            newroute.node[i] = j;
                            routes.insertElementAt( newroute, r );
                        }
                        pos++;
                    }
                r = r + pos;
            }
        }
        if ( DEBUG )
            System.out.println( "Between " + a + " and " + b + " there are " +
                                routes.size() + " different routes" );
        status = true;
        return;
    }

}


